@@ -1,5 +1,5 @@
 #
-# demo.tcl,v 1.114 2007/12/13 20:14:02 jenglish Exp
+# demo.tcl,v 1.120 2009/01/03 18:46:13 jenglish Exp
 #
 # Tile widget set -- widget demo
 #
@@ -12,7 +12,10 @@ eval destroy [winfo children .]		;# in case script is re-sourced
 variable demodir [file dirname [info script]]
 lappend auto_path . $demodir
 package require tile
-package require keynav
+
+catch { package require keynav }
+catch { package require ttk::dialog }
+catch { package require tile::harmonize }
 
 source [file join $demodir iconlib.tcl]
 source [file join $demodir repeater.tcl]
@@ -493,6 +496,33 @@ proc combobox.pane {cbf} {
 }
 $nb add [combobox.pane $nb.combos] -text "Combobox" -underline 7
 
+### Spinbox pane.
+#
+proc spinbox.pane {f} {
+    ttk::frame $f
+    set l [list \
+	January February March April May June July August September Banana]
+
+    grid \
+	[ttk::spinbox $f.tsbv -values $l -textvariable V(M) -wrap 1] \
+	[spinbox $f.sbv -values $l -textvariable V(M) -wrap 1] \
+    -sticky we ;
+
+    grid \
+	[ttk::spinbox $f.tsbl -from 0 -to 100 -wrap 1] \
+	[spinbox $f.sbl -from 0 -to 100 -wrap 1] \
+    -sticky we ;
+
+    grid columnconfigure $f 0 -weight 1
+    grid columnconfigure $f 1 -weight 1
+    grid rowconfigure $f 99 -weight 1
+
+    return $f
+}
+if {[llength [info commands ttk::spinbox]]} {
+    $nb add [spinbox.pane $nb.spinbox] -text "Spinboxes"
+}
+
 ### Treeview widget demo pane.
 #
 proc tree.pane {w} {
@@ -510,6 +540,7 @@ proc tree.pane {w} {
     populateTree $w.t .
 
     bind $w.t <KeyPress-F5> [list refreshTree %W]
+    bind $w.t <KeyPress-F6> [list unfoldTree %W]
 
     return $w
 }
@@ -543,6 +574,15 @@ proc refreshTree {tv} {
     $tv item [populateTree $tv $id] -open true
 }
 
+# unfoldTree --
+#
+proc unfoldTree {tv {item {}}} {
+    $tv item $item -open true
+    foreach child [$tv children $item] {
+	unfoldTree $tv $child
+    }
+}
+
 $nb add [tree.pane $nb.tree] -text "Tree" -sticky news
 
 ### Other demos.
@@ -669,7 +709,7 @@ proc TrackWidget {w} {
 
 variable states [list \
     active disabled focus pressed selected readonly \
-    background alternate invalid]
+    background alternate invalid hover]
 
 proc trackStates {} {
     variable states
@@ -747,7 +787,7 @@ proc repeatDemo {} {
     if {![catch { wm deiconify $top ; raise $top }]} { return }
     toplevel $top
     wm title $top "Repeating button"
-    keynav::enableMnemonics $top
+    catch { keynav::enableMnemonics $top }
 
     set f [ttk::frame .repeatDemo.f]
     ttk::button $f.b -class Repeater -text "Press and hold" \
@@ -803,7 +843,7 @@ pack [ttk::sizegrip $statusbar.grip] -side right -anchor se
 #
 bind $::ROOT <KeyPress-Escape>	[list event generate $cmd.close <<Invoke>>]
 bind $::ROOT <<Help>>		[list event generate $cmd.help <<Invoke>>]
-keynav::enableMnemonics $::ROOT
+catch { keynav::enableMnemonics $::ROOT }
 
 ### Menubar.
 #
