@@ -1,10 +1,9 @@
 /*
- * clamTheme.c,v 1.30 2007/12/02 04:34:30 jenglish Exp
+ * clamTheme.c,v 1.34 2010/11/13 18:00:16 jenglish Exp
  *
  * Copyright (C) 2004 Joe English
  *
- * Tile widget set: another theme engine.
- * Inspired by the XFCE family of Gnome themes.
+ * "clam" theme; inspired by the XFCE family of Gnome themes.
  */
 
 #include <tk.h>
@@ -120,7 +119,7 @@ static Ttk_ElementOptionSpec BorderElementOptions[] = {
 	Tk_Offset(BorderElement,reliefObj), "flat" },
     { "-borderwidth", TK_OPTION_PIXELS,
 	Tk_Offset(BorderElement,borderWidthObj), "2" },
-    {0,0,0}
+    { NULL, 0, 0, NULL }
 };
 
 /*
@@ -208,7 +207,7 @@ static Ttk_ElementOptionSpec FieldElementOptions[] = {
 	Tk_Offset(FieldElement,darkColorObj), DARK_COLOR },
     { "-fieldbackground", TK_OPTION_BORDER,
 	Tk_Offset(FieldElement,backgroundObj), "white" },
-    {0,0,0}
+    { NULL, 0, 0, NULL }
 };
 
 static void FieldElementSize(
@@ -294,7 +293,7 @@ static Ttk_ElementOptionSpec IndicatorElementOptions[] = {
 	Tk_Offset(IndicatorElement,upperColorObj), DARKEST_COLOR },
     { "-lowerbordercolor", TK_OPTION_COLOR,
 	Tk_Offset(IndicatorElement,lowerColorObj), DARK_COLOR },
-    {0,0,0}
+    { NULL, 0, 0, NULL }
 };
 
 static void IndicatorElementSize(
@@ -413,7 +412,7 @@ static Ttk_ElementOptionSpec MenuIndicatorElementOptions[] =
     { "-arrowpadding",TK_OPTION_STRING,
 	Tk_Offset(MenuIndicatorElement,paddingObj),
 	"3" },
-    { NULL }
+    { NULL, 0, 0, NULL }
 };
 
 static void MenuIndicatorElementSize(
@@ -479,7 +478,7 @@ static Ttk_ElementOptionSpec GripElementOptions[] = {
 	Tk_Offset(GripElement,borderColorObj), DARKEST_COLOR },
     { "-gripcount", TK_OPTION_INT,
 	Tk_Offset(GripElement,gripCountObj), "5" },
-    {0,0,0}
+    { NULL, 0, 0, NULL }
 };
 
 static void GripElementSize(
@@ -488,7 +487,7 @@ static void GripElementSize(
 {
     int horizontal = *((Ttk_Orient*)clientData) == TTK_ORIENT_HORIZONTAL;
     GripElement *grip = elementRecord;
-    int gripCount;
+    int gripCount = 0;
 
     Tcl_GetIntFromObj(NULL, grip->gripCountObj, &gripCount);
     if (horizontal) {
@@ -507,8 +506,8 @@ static void GripElementDraw(
     GripElement *grip = elementRecord;
     GC lightGC = Ttk_GCForColor(tkwin,grip->lightColorObj,d);
     GC darkGC = Ttk_GCForColor(tkwin,grip->borderColorObj,d);
-    int gripPad = 1;
-    int i, gripCount;
+    int gripPad = 1, gripCount = 0;
+    int i;
 
     Tcl_GetIntFromObj(NULL, grip->gripCountObj, &gripCount);
 
@@ -578,7 +577,7 @@ static Ttk_ElementOptionSpec ScrollbarElementOptions[] = {
 	Tk_Offset(ScrollbarElement,gripCountObj), "5" },
     { "-sliderlength", TK_OPTION_INT,
 	Tk_Offset(ScrollbarElement,sliderlengthObj), "30" },
-    {0,0,0}
+    { NULL, 0, 0, NULL }
 };
 
 static void TroughElementDraw(
@@ -615,7 +614,7 @@ static void ThumbElementDraw(
     Drawable d, Ttk_Box b, unsigned state)
 {
     ScrollbarElement *sb = elementRecord;
-    int gripCount = 3, orient = TTK_ORIENT_HORIZONTAL;
+    int gripCount = 0, orient = TTK_ORIENT_HORIZONTAL;
     GC lightGC, darkGC;
     int x1, y1, x2, y2, dx, dy, i;
     const int w = WIN32_XDRAWLINE_HACK;
@@ -804,7 +803,7 @@ static Ttk_ElementOptionSpec NotebookElementOptions[] = {
 	Tk_Offset(NotebookElement,lightColorObj), LIGHT_COLOR },
     { "-darkcolor", TK_OPTION_COLOR,
 	Tk_Offset(NotebookElement,darkColorObj), DARK_COLOR },
-    {0,0,0}
+    { NULL, 0, 0, NULL }
 };
 
 static void TabElementSize(
@@ -919,7 +918,7 @@ TTK_END_LAYOUT_TABLE
  * +++ Initialization.
  */
 
-int DLLEXPORT
+MODULE_SCOPE int
 TtkClamTheme_Init(Tcl_Interp *interp)
 {
     Ttk_Theme theme = Ttk_CreateTheme(interp, "clam", 0);
