@@ -1,10 +1,9 @@
-/* classicTheme.c,v 1.12 2007/12/02 04:34:30 jenglish Exp
+/*
+ * classicTheme.c,v 1.16 2010/11/13 18:00:16 jenglish Exp
  *
  * Copyright (c) 2004, Joe English
  *
- * Tile widget set: classic theme.
- *
- * Implements the "classic" Motif-like Tk look.
+ * "classic" theme; implements the classic Motif-like Tk look.
  *
  */
 
@@ -31,7 +30,7 @@ static Ttk_ElementOptionSpec HighlightElementOptions[] = {
 	Tk_Offset(HighlightElement,highlightColorObj), DEFAULT_BACKGROUND },
     { "-highlightthickness",TK_OPTION_PIXELS,
 	Tk_Offset(HighlightElement,highlightThicknessObj), "0" },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static void HighlightElementSize(
@@ -71,7 +70,7 @@ static Ttk_ElementSpec HighlightElementSpec =
 
 /*------------------------------------------------------------------------
  * +++ Button Border element:
- * 
+ *
  * The Motif-style button border on X11 consists of (from outside-in):
  *
  * + focus indicator (controlled by -highlightcolor and -highlightthickness),
@@ -88,15 +87,15 @@ typedef struct {
 
 static Ttk_ElementOptionSpec ButtonBorderElementOptions[] =
 {
-    { "-background", TK_OPTION_BORDER, 
+    { "-background", TK_OPTION_BORDER,
 	Tk_Offset(ButtonBorderElement,borderObj), DEFAULT_BACKGROUND },
-    { "-borderwidth", TK_OPTION_PIXELS, 
+    { "-borderwidth", TK_OPTION_PIXELS,
 	Tk_Offset(ButtonBorderElement,borderWidthObj), DEFAULT_BORDERWIDTH },
-    { "-relief", TK_OPTION_RELIEF, 
+    { "-relief", TK_OPTION_RELIEF,
 	Tk_Offset(ButtonBorderElement,reliefObj), "flat" },
-    { "-default", TK_OPTION_ANY, 
+    { "-default", TK_OPTION_ANY,
 	Tk_Offset(ButtonBorderElement,defaultStateObj), "disabled" },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static void ButtonBorderElementSize(
@@ -118,7 +117,7 @@ static void ButtonBorderElementSize(
 
 /*
  * (@@@ Note: ButtonBorderElement still still still buggy:
- * padding for default ring is drawn in the wrong color 
+ * padding for default ring is drawn in the wrong color
  * when the button is active.)
  */
 static void ButtonBorderElementDraw(
@@ -209,7 +208,7 @@ static Ttk_ElementOptionSpec ArrowElementOptions[] =
     { "-borderwidth", TK_OPTION_PIXELS, Tk_Offset(ArrowElement,borderWidthObj),
     	DEFAULT_BORDERWIDTH },
     { "-relief", TK_OPTION_RELIEF, Tk_Offset(ArrowElement,reliefObj),"raised" },
-    { NULL }
+    { NULL, 0, 0, NULL }
 };
 
 static void ArrowElementSize(
@@ -229,17 +228,15 @@ static void ArrowElementDraw(
 {
     int direction = *(int *)clientData;
     ArrowElement *arrow = elementRecord;
-    Tk_3DBorder border;
-    int borderWidth;
+    Tk_3DBorder border = Tk_Get3DBorderFromObj(tkwin, arrow->borderObj);
+    int borderWidth = 2;
     int relief = TK_RELIEF_RAISED;
-    int size;
+    int size = b.width < b.height ? b.width : b.height;
     XPoint points[3];
 
     Tk_GetPixelsFromObj(NULL, tkwin, arrow->borderWidthObj, &borderWidth);
-    border = Tk_Get3DBorderFromObj(tkwin, arrow->borderObj);
     Tk_GetReliefFromObj(NULL, arrow->reliefObj, &relief);
 
-    size = b.width < b.height ? b.width : b.height;
 
     /*
      * @@@ There are off-by-one pixel errors in the way these are drawn;
@@ -286,19 +283,19 @@ static Ttk_ElementSpec ArrowElementSpec =
 /*------------------------------------------------------------------------
  * +++ Sash element (for ttk::panedwindow)
  *
- * NOTES: 
+ * NOTES:
  *
  * panedwindows with -orient horizontal use vertical sashes, and vice versa.
  *
  * Interpretation of -sashrelief 'groove' and 'ridge' are
  * swapped wrt. the core panedwindow, which (I think) has them backwards.
  *
- * Default -sashrelief is sunken; the core panedwindow has default 
+ * Default -sashrelief is sunken; the core panedwindow has default
  * -sashrelief raised, but that looks wrong to me.
  */
 
 static Ttk_Orient SashClientData[] = {
-    TTK_ORIENT_HORIZONTAL, TTK_ORIENT_VERTICAL 
+    TTK_ORIENT_HORIZONTAL, TTK_ORIENT_VERTICAL
 };
 
 typedef struct {
@@ -311,19 +308,19 @@ typedef struct {
 } SashElement;
 
 static Ttk_ElementOptionSpec SashOptions[] = {
-    { "-background", TK_OPTION_BORDER, 
+    { "-background", TK_OPTION_BORDER,
 	Tk_Offset(SashElement,borderObj), DEFAULT_BACKGROUND },
-    { "-sashrelief", TK_OPTION_RELIEF, 
+    { "-sashrelief", TK_OPTION_RELIEF,
 	Tk_Offset(SashElement,sashReliefObj), "sunken" },
     { "-sashthickness", TK_OPTION_PIXELS,
 	Tk_Offset(SashElement,sashThicknessObj), "6" },
-    { "-sashpad", TK_OPTION_PIXELS, 
+    { "-sashpad", TK_OPTION_PIXELS,
 	Tk_Offset(SashElement,sashPadObj), "2" },
     { "-handlesize", TK_OPTION_PIXELS,
 	Tk_Offset(SashElement,handleSizeObj), "8" },
     { "-handlepad", TK_OPTION_PIXELS,
 	Tk_Offset(SashElement,handlePadObj), "8" },
-    {0,0,0}
+    { NULL, 0, 0, NULL }
 };
 
 static void SashElementSize(
@@ -372,10 +369,10 @@ static void SashElementDraw(
 	    gc1 = Tk_3DBorderGC(tkwin, border, TK_3D_DARK_GC);
 	    gc2 = Tk_3DBorderGC(tkwin, border, TK_3D_LIGHT_GC);
 	    break;
-	case TK_RELIEF_SOLID: 
+	case TK_RELIEF_SOLID:
 	    gc1 = gc2 = Tk_3DBorderGC(tkwin, border, TK_3D_DARK_GC);
 	    break;
-	case TK_RELIEF_FLAT: 
+	case TK_RELIEF_FLAT:
 	default:
 	    gc1 = gc2 = Tk_3DBorderGC(tkwin, border, TK_3D_FLAT_GC);
 	    break;
@@ -403,7 +400,7 @@ static void SashElementDraw(
 	    hb = Ttk_StickBox(b, handleSize, handleSize, TTK_STICK_N);
 	    hb.y += handlePad;
 	}
-	Tk_Fill3DRectangle(tkwin, d, border, 
+	Tk_Fill3DRectangle(tkwin, d, border,
 	    hb.x, hb.y, hb.width, hb.height, 1, TK_RELIEF_RAISED);
     }
 }
@@ -472,7 +469,7 @@ TTK_END_LAYOUT_TABLE
  * 	Install classic theme.
  */
 
-int TtkClassicTheme_Init(Tcl_Interp *interp)
+MODULE_SCOPE int TtkClassicTheme_Init(Tcl_Interp *interp)
 {
     Ttk_Theme theme =  Ttk_CreateTheme(interp, "classic", NULL);
 
@@ -500,7 +497,7 @@ int TtkClassicTheme_Init(Tcl_Interp *interp)
     Ttk_RegisterElement(interp, theme, "arrow",
 	    &ArrowElementSpec, &ArrowElements[0]);
 
-    Ttk_RegisterElement(interp, theme, "hsash", 
+    Ttk_RegisterElement(interp, theme, "hsash",
 	    &SashElementSpec, &SashClientData[0]);
     Ttk_RegisterElement(interp, theme, "vsash",
 	    &SashElementSpec, &SashClientData[1]);
