@@ -29,16 +29,3 @@
 	srcy, width, height);
 #endif
 
-#if defined(WIN_TK) && defined(NO_PRIVATE_HEADERS)
-#include <windows.h>
-#include <tkPlatDecls.h>
-/* From tkWinInt.h: */
-typedef struct TkWinDCState {
-    HPALETTE palette;
-    int bkmode;
-} TkWinDCState;
-
-EXTERN HDC TkWinGetDrawableDC(Display *, Drawable, TkWinDCState*);
-EXTERN HDC TkWinReleaseDrawableDC(Drawable, HDC, TkWinDCState*);
-#endif
-
