@@ -1,4 +1,4 @@
-/* label.c,v 1.22 2007/11/19 01:57:21 jenglish Exp
+/* label.c,v 1.25 2010/11/13 18:00:16 jenglish Exp
  *
  * text, image, and label elements.
  *
@@ -67,7 +67,7 @@ static Ttk_ElementOptionSpec TextElementOptions[] = {
 	Tk_Offset(TextElement,wrapLengthObj), "0" },
     { "-embossed", TK_OPTION_INT,
 	Tk_Offset(TextElement,embossedObj), "0"},
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static int TextSetup(TextElement *text, Tk_Window tkwin)
@@ -236,7 +236,7 @@ static Ttk_ElementOptionSpec ImageElementOptions[] = {
 	Tk_Offset(ImageElement,stippleObj), "gray50" },
     { "-background", TK_OPTION_COLOR,
 	Tk_Offset(ImageElement,backgroundObj), DEFAULT_BACKGROUND },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 /*
@@ -314,6 +314,12 @@ static void ImageDraw(
 	height = Tk_Height(tkwin) - b.y;
     }
 
+    if (height <= 0 || width <= 0) {
+	/* Completely clipped - bail out.
+	 */
+	return;
+    }
+
     Tk_RedrawImage(image->tkimg, 0,0, width, height, d, b.x, b.y);
 
     /* If we're disabled there's no state-specific 'disabled' image, 
@@ -446,8 +452,7 @@ static Ttk_ElementOptionSpec LabelElementOptions[] = {
 	Tk_Offset(LabelElement,image.stippleObj), "gray50" },
     { "-background", TK_OPTION_COLOR,
 	Tk_Offset(LabelElement,image.backgroundObj), DEFAULT_BACKGROUND },
-
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 /*
@@ -652,6 +657,7 @@ static Ttk_ElementSpec LabelElementSpec = {
  * +++ Initialization.
  */
 
+MODULE_SCOPE
 void TtkLabel_Init(Tcl_Interp *interp)
 {
     Ttk_Theme theme =  Ttk_GetDefaultTheme(interp);
