@@ -1,4 +1,4 @@
-/* separator.c,v 1.10 2007/10/25 06:42:47 jenglish Exp
+/* separator.c,v 1.12 2010/11/13 18:00:16 jenglish Exp
  *
  * Copyright (c) 2004, Joe English
  *
@@ -49,14 +49,13 @@ static Ttk_Layout SeparatorGetLayout(
 /*
  * Widget commands:
  */
-static WidgetCommandSpec SeparatorCommands[] =
-{
-    { "configure",	TtkWidgetConfigureCommand },
-    { "cget",		TtkWidgetCgetCommand },
-    { "identify",	TtkWidgetIdentifyCommand },
-    { "instate",	TtkWidgetInstateCommand },
-    { "state",  	TtkWidgetStateCommand },
-    { NULL, NULL }
+static const Ttk_Ensemble SeparatorCommands[] = {
+    { "configure",	TtkWidgetConfigureCommand,0 },
+    { "cget",		TtkWidgetCgetCommand,0 },
+    { "identify",	TtkWidgetIdentifyCommand,0 },
+    { "instate",	TtkWidgetInstateCommand,0 },
+    { "state",  	TtkWidgetStateCommand,0 },
+    { 0,0,0 }
 };
 
 /*
@@ -86,14 +85,13 @@ TTK_END_LAYOUT
  * 	Has no options or methods other than the standard ones.
  */
 
-static WidgetCommandSpec SizegripCommands[] =
-{
-    { "configure",	TtkWidgetConfigureCommand },
-    { "cget",		TtkWidgetCgetCommand },
-    { "identify",	TtkWidgetIdentifyCommand },
-    { "instate",	TtkWidgetInstateCommand },
-    { "state",  	TtkWidgetStateCommand },
-    { NULL, NULL }
+static const Ttk_Ensemble SizegripCommands[] = {
+    { "configure",	TtkWidgetConfigureCommand,0 },
+    { "cget",		TtkWidgetCgetCommand,0 },
+    { "identify",	TtkWidgetIdentifyCommand,0 },
+    { "instate",	TtkWidgetInstateCommand,0 },
+    { "state",  	TtkWidgetStateCommand,0 },
+    { 0,0,0 }
 };
 
 static WidgetSpec SizegripWidgetSpec =
@@ -118,6 +116,8 @@ TTK_END_LAYOUT
 
 /* +++ Initialization:
  */
+
+MODULE_SCOPE
 void TtkSeparator_Init(Tcl_Interp *interp)
 {
     Ttk_Theme theme = Ttk_GetDefaultTheme(interp);
