@@ -1,4 +1,4 @@
-/* tkElements.c,v 1.85 2007/11/19 01:57:21 jenglish Exp
+/* tkElements.c,v 1.88 2010/11/13 18:00:16 jenglish Exp
  *
  * Copyright (c) 2003, Joe English
  *
@@ -21,7 +21,7 @@
  * and may be used in other engines.
  */
 
-/* public */ Ttk_ElementOptionSpec TtkNullElementOptions[] = { {NULL} };
+/* public */ Ttk_ElementOptionSpec TtkNullElementOptions[] = { { NULL, 0, 0, NULL } };
 
 /* public */ void
 TtkNullElementSize(
@@ -61,7 +61,7 @@ typedef struct {
 static Ttk_ElementOptionSpec BackgroundElementOptions[] = {
     { "-background", TK_OPTION_BORDER,
 	    Tk_Offset(BackgroundElement,backgroundObj), DEFAULT_BACKGROUND },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static void FillElementDraw(
@@ -118,7 +118,7 @@ static Ttk_ElementOptionSpec BorderElementOptions[] = {
 	Tk_Offset(BorderElement,borderWidthObj), DEFAULT_BORDERWIDTH },
     { "-relief", TK_OPTION_RELIEF,
 	Tk_Offset(BorderElement,reliefObj), "flat" },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static void BorderElementSize(
@@ -171,7 +171,7 @@ static Ttk_ElementOptionSpec FieldElementOptions[] = {
 	Tk_Offset(FieldElement,borderObj), "white" },
     { "-borderwidth", TK_OPTION_PIXELS,
 	Tk_Offset(FieldElement,borderWidthObj), "2" },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static void FieldElementSize(
@@ -229,7 +229,7 @@ static Ttk_ElementOptionSpec PaddingElementOptions[] = {
 	Tk_Offset(PaddingElement,reliefObj), "flat" },
     { "-shiftrelief", TK_OPTION_INT,
 	Tk_Offset(PaddingElement,shiftreliefObj), "0" },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static void PaddingElementSize(
@@ -293,7 +293,7 @@ static Ttk_ElementOptionSpec FocusElementOptions[] = {
 	Tk_Offset(FocusElement,focusColorObj), "black" },
     { "-focusthickness",TK_OPTION_PIXELS,
 	Tk_Offset(FocusElement,focusThicknessObj), "1" },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static void FocusElementSize(
@@ -345,7 +345,7 @@ static Ttk_ElementOptionSpec SeparatorElementOptions[] = {
 	Tk_Offset(SeparatorElement, orientObj), "horizontal" },
     { "-background", TK_OPTION_BORDER,
 	Tk_Offset(SeparatorElement,borderObj), DEFAULT_BACKGROUND },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 static void SeparatorElementSize(
@@ -506,7 +506,7 @@ static Ttk_ElementOptionSpec IndicatorElementOptions[] = {
 	Tk_Offset(IndicatorElement,marginObj), "0 2 4 2" },
     { "-borderwidth", TK_OPTION_PIXELS,
 	Tk_Offset(IndicatorElement,borderWidthObj), DEFAULT_BORDERWIDTH },
-    {NULL}
+    { NULL, 0, 0, NULL }
 };
 
 /*
@@ -652,7 +652,7 @@ static Ttk_ElementOptionSpec MenuIndicatorElementOptions[] = {
 	Tk_Offset(MenuIndicatorElement,reliefObj),"raised" },
     { "-indicatormargin", TK_OPTION_STRING,
 	    Tk_Offset(MenuIndicatorElement,marginObj), "5 0" },
-    { NULL }
+    { NULL, 0, 0, NULL }
 };
 
 static void MenuIndicatorElementSize(
@@ -719,7 +719,7 @@ static Ttk_ElementOptionSpec ArrowElementOptions[] = {
 	Tk_Offset(ArrowElement,colorObj),"black"},
     { "-arrowsize", TK_OPTION_PIXELS,
 	Tk_Offset(ArrowElement,sizeObj), "14" },
-    { NULL }
+    { NULL, 0, 0, NULL }
 };
 
 static Ttk_Padding ArrowMargins = { 3,3,3,3 };
@@ -786,7 +786,7 @@ static Ttk_ElementOptionSpec TroughElementOptions[] = {
 	Tk_Offset(TroughElement,colorObj), DEFAULT_BACKGROUND },
     { "-troughrelief",TK_OPTION_RELIEF,
 	Tk_Offset(TroughElement,reliefObj), "sunken" },
-    { NULL }
+    { NULL, 0, 0, NULL }
 };
 
 static void TroughElementSize(
@@ -850,7 +850,7 @@ static Ttk_ElementOptionSpec ThumbElementOptions[] = {
 	Tk_Offset(ThumbElement,borderObj), DEFAULT_BACKGROUND },
     { "-borderwidth", TK_OPTION_PIXELS,
 	Tk_Offset(ThumbElement,borderWidthObj), DEFAULT_BORDERWIDTH },
-    { NULL }
+    { NULL, 0, 0, NULL }
 };
 
 static void ThumbElementSize(
@@ -923,7 +923,7 @@ static Ttk_ElementOptionSpec SliderElementOptions[] = {
 	DEFAULT_BACKGROUND },
     { "-orient", TK_OPTION_ANY, Tk_Offset(SliderElement,orientObj),
 	"horizontal" },
-    { NULL }
+    { NULL, 0, 0, NULL }
 };
 
 static void SliderElementSize(
@@ -956,7 +956,7 @@ static void SliderElementDraw(
 {
     SliderElement *slider = elementRecord;
     Tk_3DBorder border = NULL;
-    int relief, borderWidth, orient;
+    int relief = TK_RELIEF_RAISED, borderWidth = 2, orient;
 
     border = Tk_Get3DBorderFromObj(tkwin, slider->borderObj);
     Ttk_GetOrientFromObj(NULL, slider->orientObj, &orient);
@@ -1034,7 +1034,7 @@ static Ttk_ElementOptionSpec PbarElementOptions[] = {
 	DEFAULT_BORDERWIDTH },
     { "-background", TK_OPTION_BORDER, Tk_Offset(PbarElement,borderObj),
 	DEFAULT_BACKGROUND },
-    { NULL }
+    { NULL, 0, 0, NULL }
 };
 
 static void PbarElementSize(
@@ -1042,7 +1042,7 @@ static void PbarElementSize(
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     PbarElement *pbar = elementRecord;
-    int orient, thickness, length, borderWidth;
+    int orient, thickness = 15, length = 30, borderWidth = 2;
 
     Ttk_GetOrientFromObj(NULL, pbar->orientObj, &orient);
     Tk_GetPixelsFromObj(NULL, tkwin, pbar->thicknessObj, &thickness);
@@ -1067,7 +1067,7 @@ static void PbarElementDraw(
 {
     PbarElement *pbar = elementRecord;
     Tk_3DBorder border = Tk_Get3DBorderFromObj(tkwin, pbar->borderObj);
-    int relief, borderWidth;
+    int relief = TK_RELIEF_RAISED, borderWidth = 2;
 
     Tk_GetPixelsFromObj(NULL, tkwin, pbar->borderWidthObj, &borderWidth);
     Tk_GetReliefFromObj(NULL, pbar->reliefObj, &relief);
@@ -1217,6 +1217,7 @@ static Ttk_ElementSpec ClientElementSpec = {
  *	Register default element implementations.
  */
 
+MODULE_SCOPE
 void TtkElements_Init(Tcl_Interp *interp)
 {
     Ttk_Theme theme =  Ttk_GetDefaultTheme(interp);
