@@ -1,4 +1,4 @@
-/* trace.c,v 1.4 2005/09/12 03:11:01 jenglish Exp
+/* trace.c,v 1.5 2008/05/23 20:32:44 jenglish Exp
  *
  * Copyright 2003, Joe English
  *
@@ -27,8 +27,8 @@ static char *
 VarTraceProc(
     ClientData clientData,	/* Widget record pointer */
     Tcl_Interp *interp, 	/* Interpreter containing variable. */
-    CONST char *name1,		/* (unused) */
-    CONST char *name2,		/* (unused) */
+    const char *name1,		/* (unused) */
+    const char *name2,		/* (unused) */
     int flags)			/* Information about what happened. */
 {
     Ttk_TraceHandle *tracePtr = clientData;
