@@ -1,5 +1,5 @@
 #
-# altTheme.tcl,v 1.40 2007/12/08 03:52:01 jenglish Exp
+# altTheme.tcl,v 1.44 2008/12/31 21:25:01 jenglish Exp
 #
 # Tile widget set: Alternate theme
 #
@@ -11,6 +11,7 @@ namespace eval ttk::theme::alt {
 	-frame 		"#d9d9d9"
 	-window		"#ffffff"
 	-darker 	"#c3c3c3"
+	-border		"#414141"
 	-activebg 	"#ececec"
 	-disabledfg	"#a3a3a3"
 	-selectbg	"#4a6984"
@@ -23,6 +24,7 @@ namespace eval ttk::theme::alt {
 	    -background 	$colors(-frame) \
 	    -foreground 	black \
 	    -troughcolor	$colors(-darker) \
+	    -bordercolor	$colors(-border) \
 	    -selectbackground 	$colors(-selectbg) \
 	    -selectforeground 	$colors(-selectfg) \
 	    -font 		TkDefaultFont \
@@ -59,6 +61,13 @@ namespace eval ttk::theme::alt {
 	ttk::style configure TCombobox -padding 1
 	ttk::style map TCombobox -fieldbackground \
 		[list readonly $colors(-frame) disabled $colors(-frame)]
+	ttk::style configure ComboboxPopdownFrame \
+	    -relief solid -borderwidth 1
+
+	ttk::style configure TSpinbox -arrowsize 10 -padding {2 0 10 0}
+	ttk::style map TSpinbox -fieldbackground \
+	    [list readonly $colors(-frame) disabled $colors(-frame)] \
+	    -arrowcolor [list disabled $colors(-disabledfg)]
 
 	ttk::style configure Toolbutton -relief flat -padding 2
 	ttk::style map Toolbutton -relief \
@@ -80,15 +89,8 @@ namespace eval ttk::theme::alt {
 
 	# Treeview:
 	ttk::style configure Heading -font TkHeadingFont -relief raised
-	ttk::style configure Row -background $colors(-window)
-	ttk::style configure Cell -background $colors(-window)
-	ttk::style map Row \
-	    -background [list selected $colors(-selectbg)] \
-	    -foreground [list selected $colors(-selectfg)] ;
-	ttk::style map Cell \
-	    -background [list selected $colors(-selectbg)] \
-	    -foreground [list selected $colors(-selectfg)] ;
-	ttk::style map Item \
+	ttk::style configure Treeview -background $colors(-window)
+	ttk::style map Treeview \
 	    -background [list selected $colors(-selectbg)] \
 	    -foreground [list selected $colors(-selectfg)] ;
 
