@@ -1,5 +1,5 @@
 #
-# button.tcl,v 1.14 2006/11/26 17:45:16 jenglish Exp
+# button.tcl,v 1.15 2009/11/12 18:18:00 jenglish Exp
 #
 # Bindings for Buttons, Checkbuttons, and Radiobuttons.
 #
@@ -54,7 +54,7 @@ bind TRadiobutton <KeyPress-Down> 	{ ttk::button::RadioTraverse %W +1 }
 proc ttk::button::activate {w} {
     $w instate disabled { return }
     set oldState [$w state pressed]
-    update idletasks; after 100
+    update idletasks; after 100	;# block event loop to avoid reentrancy
     $w state $oldState
     $w invoke
 }
