@@ -1,5 +1,5 @@
 #
-# paned.tcl,v 1.5 2007/06/09 21:45:51 jenglish Exp
+# paned.tcl,v 1.6 2008/10/28 19:38:02 jenglish Exp
 #
 # Bindings for ttk::panedwindow widget.
 #
@@ -23,11 +23,10 @@ bind TPanedwindow <ButtonRelease-1> 	{ ttk::panedwindow::Release %W %x %y }
 
 bind TPanedwindow <Motion> 		{ ttk::panedwindow::SetCursor %W %x %y }
 bind TPanedwindow <Enter> 		{ ttk::panedwindow::SetCursor %W %x %y }
-bind TPanedwindow <Leave> 		{ ttk::panedwindow::ResetCursor %W } 
+bind TPanedwindow <Leave> 		{ ttk::panedwindow::ResetCursor %W }
 # See <<NOTE-PW-LEAVE-NOTIFYINFERIOR>>
 bind TPanedwindow <<EnteredChild>>	{ ttk::panedwindow::ResetCursor %W }
 
-
 ## Sash movement:
 #
 proc ttk::panedwindow::Press {w x y} {
@@ -36,7 +35,7 @@ proc ttk::panedwindow::Press {w x y} {
     set sash [$w identify $x $y]
     if {$sash eq ""} {
     	set State(pressed) 0
-	return 
+	return
     }
     set State(pressed) 	1
     set State(pressX) 	$x
@@ -66,22 +65,20 @@ proc ttk::panedwindow::Release {w x y} {
 proc ttk::panedwindow::ResetCursor {w} {
     variable State
     if {!$State(pressed)} {
-	$w configure -cursor {}
+	ttk::setCursor $w {}
     }
 }
 
 proc ttk::panedwindow::SetCursor {w x y} {
-    variable ::ttk::Cursors
-
-    if {![llength [$w identify $x $y]]} {
-    	ResetCursor $w
-    } else {
+    set cursor ""
+    if {[llength [$w identify $x $y]]} {
     	# Assume we're over a sash.
 	switch -- [$w cget -orient] {
-	    horizontal 	{ $w configure -cursor $Cursors(hresize) }
-	    vertical 	{ $w configure -cursor $Cursors(vresize) }
+	    horizontal 	{ set cursor hresize }
+	    vertical 	{ set cursor vresize }
 	}
     }
+    ttk::setCursor $w $cursor
 }
 
 #*EOF*
