@@ -1,14 +1,22 @@
 #
-# sizegrip.tcl,v 1.3 2006/11/05 18:40:48 jenglish Exp
+# sizegrip.tcl,v 1.6 2009/12/23 04:35:16 jenglish Exp
 #
-# Tile widget set -- sizegrip widget bindings.
+# Sizegrip widget bindings.
 #
 # Dragging a sizegrip widget resizes the containing toplevel.
 #
 # NOTE: the sizegrip widget must be in the lower right hand corner.
 #
 
-option add *TSizegrip.cursor $::ttk::Cursors(seresize)
+switch -- [tk windowingsystem] {
+    x11 -
+    win32 {
+	option add *TSizegrip.cursor [ttk::cursor seresize]
+    }
+    aqua {
+    	# Aqua sizegrips use default Arrow cursor.
+    }
+}
 
 namespace eval ttk::sizegrip {
     variable State
@@ -20,6 +28,8 @@ namespace eval ttk::sizegrip {
 	height 		0
 	widthInc	1
 	heightInc	1
+        resizeX         1
+        resizeY         1
 	toplevel 	{}
     }
 }
@@ -31,13 +41,21 @@ bind TSizegrip <ButtonRelease-1> 	{ ttk::sizegrip::Release %W %X %Y }
 proc ttk::sizegrip::Press {W X Y} {
     variable State
 
+    if {[$W instate disabled]} { return }
+
     set top [winfo toplevel $W]
 
+    # If the toplevel is not resizable then bail
+    foreach {State(resizeX) State(resizeY)} [wm resizable $top] break
+    if {!$State(resizeX) && !$State(resizeY)} {
+        return
+    }
+
     # Sanity-checks:
     #	If a negative X or Y position was specified for [wm geometry],
     #   just bail out -- there's no way to handle this cleanly.
     #
-    if {[scan [wm geometry $top] "%dx%d+%d+%d" width height _x _y] != 4} {
+    if {[scan [wm geometry $top] "%dx%d+%d+%d" width height x y] != 4} {
 	return;
     }
 
@@ -56,17 +74,26 @@ proc ttk::sizegrip::Press {W X Y} {
     set State(pressY) $Y
     set State(width)  $width
     set State(height) $height
+    set State(x)      $x
+    set State(y)      $y
     set State(pressed) 1
 }
 
 proc ttk::sizegrip::Drag {W X Y} {
     variable State
     if {!$State(pressed)} { return }
-    set w [expr {$State(width)  + ($X - $State(pressX))/$State(widthInc)}]
-    set h [expr {$State(height) + ($Y - $State(pressY))/$State(heightInc)}]
+    set w $State(width)
+    set h $State(height)
+    if {$State(resizeX)} {
+        set w [expr {$w + ($X - $State(pressX))/$State(widthInc)}]
+    }
+    if {$State(resizeY)} {
+        set h [expr {$h + ($Y - $State(pressY))/$State(heightInc)}]
+    }
     if {$w <= 0} { set w 1 }
     if {$h <= 0} { set h 1 }
-    wm geometry $State(toplevel) ${w}x${h}
+    set x $State(x) ; set y $State(y)
+    wm geometry $State(toplevel) ${w}x${h}+${x}+${y}
 }
 
 proc ttk::sizegrip::Release {W X Y} {
