@@ -1,4 +1,4 @@
-# treeview.tcl,v 1.22 2006/12/18 15:05:59 jenglish Exp
+# treeview.tcl,v 1.28 2009/11/12 18:18:01 jenglish Exp
 #
 # ttk::treeview widget bindings and utilities.
 #
@@ -21,15 +21,6 @@ namespace eval ttk::treeview {
 
     # For pressmode == "heading"
     set State(heading)  	{}
-
-    # Provide [lassign] if not already present
-    # (@@@ TODO: check if this is still needed after horrible-identify purge)
-    #
-    if {![llength [info commands lassign]]} {
-	proc lassign {vals args} {
-	    uplevel 1 [list foreach $args $vals break]
-	}
-    }
 }
 
 ### Widget bindings.
@@ -56,13 +47,7 @@ bind Treeview	<Shift-ButtonPress-1> \
 bind Treeview	<Control-ButtonPress-1> \
 		{ ttk::treeview::Select %W %x %y toggle }
 
-# Standard mousewheel bindings:
-#
-bind Treeview <MouseWheel> { %W yview scroll [expr {- (%D / 120) * 4}] units }
-if {[string equal "x11" [tk windowingsystem]]} {
-    bind Treeview <ButtonPress-4>	{ %W yview scroll -5 units }
-    bind Treeview <ButtonPress-5>	{ %W yview scroll 5 units }
-}
+ttk::copyBindings TtkScrollable Treeview
 
 ### Binding procedures.
 #
@@ -118,21 +103,15 @@ proc ttk::treeview::Keynav {w dir} {
 #	Sets cursor, active element ...
 #
 proc ttk::treeview::Motion {w x y} {
-    variable ::ttk::Cursors
-    variable State
-
     set cursor {}
     set activeHeading {}
 
-    lassign [$w identify $x $y] what where detail
-    switch -- $what {
-	separator { set cursor $Cursors(hresize) }
-	heading { set activeHeading $where }
+    switch -- [$w identify region $x $y] {
+	separator { set cursor hresize }
+	heading { set activeHeading [$w identify column $x $y] }
     }
 
-    if {[$w cget -cursor] ne $cursor} {
-	$w configure -cursor $cursor
-    }
+    ttk::setCursor $w $cursor
     ActivateHeading $w $activeHeading
 }
 
@@ -163,7 +142,7 @@ proc ttk::treeview::Select {w x y op} {
     }
 }
 
-## DoubleClick -- Double-ButtonPress-1 binding.  
+## DoubleClick -- Double-ButtonPress-1 binding.
 #
 proc ttk::treeview::DoubleClick {w x y} {
     if {[set row [$w identify row $x $y]] ne ""} {
@@ -176,19 +155,20 @@ proc ttk::treeview::DoubleClick {w x y} {
 ## Press -- ButtonPress binding.
 #
 proc ttk::treeview::Press {w x y} {
-    lassign [$w identify $x $y] what where detail
-    focus $w	;# or: ClickToFocus?
-
-    switch -- $what {
+    focus $w
+    switch -- [$w identify region $x $y] {
 	nothing { }
-	heading { heading.press $w $where }
-	separator { resize.press $w $x $where }
-	cell -
-	row  -
-	item { SelectOp $w $where choose }
-    }
-    if {$what eq "item" && [string match *indicator $detail]} {
-    	Toggle $w $where
+	heading { heading.press $w $x $y }
+	separator { resize.press $w $x $y }
+	tree -
+	cell {
+	    set item [$w identify item $x $y]
+	    SelectOp $w $item choose
+	    switch -glob -- [$w identify element $x $y] {
+		*indicator -
+		*disclosure { Toggle $w $item }
+	    }
+	}
     }
 }
 
@@ -214,10 +194,10 @@ proc ttk::treeview::Release {w x y} {
 
 ### Interactive column resizing.
 #
-proc ttk::treeview::resize.press {w x column} {
+proc ttk::treeview::resize.press {w x y} {
     variable State
     set State(pressMode) "resize"
-    set State(resizeColumn) $column
+    set State(resizeColumn) [$w identify column $x $y]
 }
 
 proc ttk::treeview::resize.drag {w x} {
@@ -232,8 +212,9 @@ proc ttk::treeview::resize.release {w x} {
 ### Heading activation.
 #
 
-proc ttk::treeview::heading.press {w column} {
+proc ttk::treeview::heading.press {w x y} {
     variable State
+    set column [$w identify column $x $y]
     set State(pressMode) "heading"
     set State(heading) $column
     $w heading $column state pressed
@@ -241,8 +222,9 @@ proc ttk::treeview::heading.press {w column} {
 
 proc ttk::treeview::heading.drag {w x y} {
     variable State
-    lassign [$w identify $x $y] what where detail
-    if {$what eq "heading" && $where eq $State(heading)} {
+    if {   [$w identify region $x $y] eq "heading"
+        && [$w identify column $x $y] eq $State(heading)
+    } {
     	$w heading $State(heading) state pressed
     } else {
     	$w heading $State(heading) state !pressed
@@ -252,7 +234,7 @@ proc ttk::treeview::heading.drag {w x y} {
 proc ttk::treeview::heading.release {w} {
     variable State
     if {[lsearch -exact [$w heading $State(heading) state] pressed] >= 0} {
-	after idle [$w heading $State(heading) -command]
+	after 0 [$w heading $State(heading) -command]
     }
     $w heading $State(heading) state !pressed
 }
@@ -261,7 +243,7 @@ proc ttk::treeview::heading.release {w} {
 #
 
 ## SelectOp $w $item [ choose | extend | toggle ] --
-#	Dispatch to appropriate selection operation 
+#	Dispatch to appropriate selection operation
 #	depending on current value of -selectmode.
 #
 proc ttk::treeview::SelectOp {w item op} {
@@ -282,17 +264,17 @@ proc ttk::treeview::select.extend.browse {w item} { BrowseTo $w $item }
 
 ## -selectmode multiple:
 #
-proc ttk::treeview::select.choose.extended {w item} { 
+proc ttk::treeview::select.choose.extended {w item} {
     BrowseTo $w $item
 }
-proc ttk::treeview::select.toggle.extended {w item} { 
-    $w selection toggle $item
+proc ttk::treeview::select.toggle.extended {w item} {
+    $w selection toggle [list $item]
 }
 proc ttk::treeview::select.extend.extended {w item} {
     if {[set anchor [$w focus]] ne ""} {
 	$w selection set [between $w $anchor $item]
     } else {
-    	BrowseTo $item
+    	BrowseTo $w $item
     }
 }
 
@@ -304,7 +286,7 @@ proc ttk::treeview::select.extend.extended {w item} {
 #	in preorder traversal order.  $item1 and $item2 may be
 #	in either order.
 #
-# NOTES: 
+# NOTES:
 #	This routine is O(N) in the size of the tree.
 #	There's probably a way to do this that's O(N) in the number
 #	of items returned, but I'm not clever enough to figure it out.
