@@ -1,6 +1,6 @@
-# tvtest.tcl,v 1.11 2006/12/18 19:03:33 jenglish Exp
-# 
-# Sandbox test script for treeview testing 
+# tvtest.tcl,v 1.13 2008/05/23 20:05:23 jenglish Exp
+#
+# Sandbox test script for treeview testing
 #
 
 package require tile
@@ -9,11 +9,15 @@ source [file join [file dirname [info script]] testutils.tcl]
 
 set haveswaplist [expr {![catch {package require swaplist}]}]
 set haveinplace  [expr {![catch {package require tile::inplace}]}]
+set havekeynav  [expr {![catch {package require keynav}]}]
 
 option add *Toolbar*TCheckbutton.style Toolbutton
 option add *Toolbar*TRadiobutton.style Toolbutton
+option add *Toolbar*takeFocus 0
 option add *tearOff 0
 
+if {$havekeynav} { keynav::enableMnemonics . }
+
 #
 # Tree window:
 #
@@ -36,20 +40,23 @@ grid rowconfigure $f 0 -weight 1
 # Status bar:
 #
 
-style configure Indicator.TLabel -relief sunken -borderwidth 1
+global S
+
+ttk::style configure Indicator.TLabel -relief sunken -borderwidth 1
 option add *Indicator.style Indicator.TLabel
 
 set s [ttk::frame .status -class Statusbar]
 
 pack \
-    [ttk::label $s.l -class Indicator -textvariable message -width 20] \
-    [ttk::label $s.lr -text  "Row:" ] \
-    [ttk::label $s.r -class Indicator -textvariable ::irow -width 10] \
-    [ttk::label $s.lc -text  "Col:" ] \
-    [ttk::label $s.c -class Indicator -textvariable ::icol -width 10] \
-    [ttk::label $s.lb -text  "BBox:" ] \
-    [ttk::label $s.b -class Indicator -textvariable ::bbox -width 16] \
--side left -pady 2;
+    [ttk::label $s.p -class Indicator -textvariable S(coords) -width 8] \
+    [ttk::label $s.r -class Indicator -textvariable S(region) -width 10] \
+    [ttk::label $s.i -class Indicator -textvariable S(item) -width 10] \
+    [ttk::label $s.d -class Indicator -textvariable S(dcolumn) -width 4] \
+    [ttk::label $s.c -class Indicator -textvariable S(column) -width 8] \
+    [ttk::label $s.e -class Indicator -textvariable S(element) -width 16] \
+    [ttk::label $s.b -class Indicator -textvariable S(bbox) -width 16] \
+    [ttk::label $s.t -class Indicator -textvariable S(tags) -width 20] \
+-side left -pady 2 -padx 2;
 
 #
 # View control:
@@ -61,7 +68,7 @@ proc setShow {tv} {
     set show [list]
     foreach {what} [list headings tree] {
     	if {$Show($what)} {
-	    lappend show $what 
+	    lappend show $what
 	}
     }
     $tv configure -show $show
@@ -79,7 +86,7 @@ proc setSelectMode {tv} {
 #
 set m [menu .menu]
 
-$m add cascade -label View -underline 0 -menu [menu $m.view] 
+$m add cascade -label View -underline 0 -menu [menu $m.view]
 $m.view add checkbutton -label "Headings?" \
     -variable Show(headings) -command [list setShow $f.tv]
 $m.view add checkbutton -label "Tree?" \
@@ -94,6 +101,9 @@ $m.view add radiobutton -label "Browse" \
 $m.view add radiobutton -label "None" \
     -variable State(selectMode) -value none \
     -command [list setSelectMode $f.tv] ;
+$m.view add separator
+$m.view add command -label "Tags..." -underline 0 \
+    -command [list tagdialog $tv.tags] ;
 
 . configure -menu $m
 
@@ -107,7 +117,8 @@ pack \
     	-variable Show(headings) -command [list setShow $f.tv]] \
     [ttk::checkbutton .t.st -text "Tree?" \
     	-variable Show(tree) -command [list setShow $f.tv]] \
-    [ttk::button .t.db -text "Directory" -command [list dirbrowse [pwd]]] \
+    [ttk::button .t.db -text "Directory" -underline 0 \
+	-command [list dirbrowse [pwd]]] \
     -side left;
 
 if {$haveswaplist} {
@@ -131,9 +142,9 @@ $tv heading X -text "XXX"
 $tv heading Y -text "YYY"
 $tv heading Z -text "ZZZ"
 
-pack .t -side top -expand false -fill x
-pack .status -side bottom -expand false -fill x
-pack .f -side top -expand true -fill both -padx 6 -pady 6
+pack .t -side top -fill x
+pack .status -side bottom -fill x
+pack .f -side top -expand true -fill both 
 
 bind . <KeyPress-Escape> [list destroy .]
 
@@ -144,21 +155,31 @@ bind $tv <KeyPress-Delete> { %W delete [%W selection] }
 #	Update status bar
 #
 proc identify {tv x y} {
-    # Old form:
-    set ::message [$tv identify $x $y]
+    global S
 
-    # New form:
-    set ::irow [$tv identify row $x $y]
-    set ::icol [$tv identify column $x $y]
+    set S(horrible) 	[$tv identify $x $y]
+    set S(coords)	"$x,$y"
+    set S(region)	[$tv identify region $x $y]
+    set S(item)		[$tv identify item $x $y]
+    set S(dcolumn)	[$tv identify column $x $y]
+    set S(element)	[$tv identify element $x $y]
 
-    if {$::irow ne ""} {
-	if {$::icol ne ""} { 
-	    set ::bbox [$tv bbox $::irow $::icol] 
+    if {$S(dcolumn) ne ""} {
+	set S(column)	[$tv column $S(dcolumn) -id]
+    } else {
+	set S(column)	""
+    }
+
+    if {$S(item) ne ""} {
+	set S(tags) [$tv item $S(item) -tags]
+	if {$S(column) ne ""} {
+	    set S(bbox) [$tv bbox $S(item) $S(column)]
 	} else {
-	    set ::bbox [$tv bbox $::irow]
+	    set S(bbox) [$tv bbox $S(item)]
 	}
     } else {
-	set ::bbox N/A
+	set S(tags) ""
+	set S(bbox) ""
     }
 }
 
@@ -184,10 +205,9 @@ $tv insert a end -id a2 -text "Second subnode" -values [list e1 e2 e3]
 $tv insert a1 end -text "Subsubnode!!" -values [list X! Y! Z!]
 
 foreach label {a b c d e f g h i j k l m n o} {
-    $tv insert b end -text $label
+    $tv insert b end -text $label -values [list x y z]
 }
 
-
 # Directory browser:
 
 # SOURCE: tkfbox.tcl
@@ -203,12 +223,12 @@ proc loaddir {tv node dir} {
     variable Icons
     foreach subdir [glob -nocomplain -type d -directory $dir *] {
         set dirnode [$tv insert $node end -text [file tail $subdir] \
-    		-image $Icons(folder) -tags DIR -values [list X Y Z]]
+	    -tags DIR -values [list X Y Z]]
 	loaddir $tv $dirnode $subdir
     }
     foreach file [glob -nocomplain -type f -directory $dir *] {
 	$tv insert $node end -text [file tail $file] \
-	    -image $Icons(file) -tags FILE -values [list X Y Z]
+	    -tags FILE -values [list X Y Z]
     }
 }
 
@@ -220,19 +240,22 @@ proc scandir {dir} {
 }
 
 proc dirbrowse {dir} {
+    variable Icons
     set tv $::tv
-    $tv tag bind DIR <Double-ButtonPress-1> { 
+    $tv tag bind DIR <Double-ButtonPress-1> {
 	puts "DIR: [%W item [%W identify row %x %y] -text]"
     }
-    $tv tag bind FILE <Double-ButtonPress-1> { 
+    $tv tag bind FILE <Double-ButtonPress-1> {
 	puts "FILE: [%W item [%W identify row %x %y] -text]"
     }
-    $tv tag configure DIR -foreground #000077 -background #777700
-    $tv tag configure FILE -foreground #007700 -background #770077
+    $tv tag configure DIR \
+    	-foreground #0000FF -background #777700 -image $Icons(folder)
+    $tv tag configure FILE \
+    	-foreground #00FF00 -background #770077 -image $Icons(file)
+    $tv tag configure DIR -font {times 12}
     loaddir $tv [$tv insert {} end -text $dir -tags DIR] $dir
 }
 
-
 proc setWidths {tv {weights {}} {width {}}} {
     if {$width eq {}} { set width [winfo width $tv] }
     array set W $weights
@@ -257,7 +280,7 @@ proc setWidths {tv {weights {}} {width {}}} {
 
     foreach column $columns weight $weights {
 	set colwidth [expr {int(($width * $weight)/$totalWeight)}]
-	$tv column $column -width $colwidth 
+	$tv column $column -width $colwidth
     }
 }
 
@@ -330,3 +353,99 @@ proc column-menu/stretch-changed {m} {
     upvar #0 $m M
     $M(tv) column $M(column) -stretch $M(stretch)
 }
+
+### Tag manipulation dialog:
+#
+namespace eval tagdialog {}
+
+proc tagdialog {t} {
+    if {[winfo exists $t]} {
+	wm deiconify $t; raise $t; return $t;
+    }
+    tagdialog::create $t
+}
+
+### 
+proc tagdialog::create {t} {
+    upvar #0 $t T
+
+    toplevel $t ; wm withdraw $t
+    wm title $t "Tags..."
+    wm transient $t [winfo toplevel [winfo parent $t]]
+
+    set T(treeview) [winfo parent $t]
+    set T(options) {-text -image -background -foreground -font}
+    # ALSO: -anchor, but can't take "-anchor {}"
+
+    set f [ttk::frame $t.f -padding 6]
+
+    # Tag selector.
+    #
+    ttk::combobox $f.tag \
+	-textvariable ${t}(tag) -postcommand [list tagdialog::cbpost $t $f.tag]
+    bind $f.tag <<ComboboxSelected>> "[list tagdialog::Refresh $t] ; break"
+    bind $f.tag <KeyPress-Return> "[list tagdialog::Refresh $t] ; break"
+
+    # Tag options.
+    # 
+    set o [ttk::frame $f.options]
+    grid [ttk::label $o.ltag -text tag: -anchor e] \
+	-row 0 -column 0 -sticky ew -padx 3
+    grid $f.tag -in $o -row 0 -column 1 -sticky ew -pady 4
+    raise $f.tag
+    foreach option $T(options) {
+	grid \
+	    [ttk::label $o.l$option -text "$option:" -anchor e] \
+	    [ttk::entry $o.v$option -textvariable ${t}($option)] \
+	-sticky ew -padx 3
+    }
+    grid columnconfigure $o 1 -weight 1
+
+    # Command box.
+    #
+    set cmd [ttk::frame $f.cmd]
+    grid x \
+    	[ttk::button $cmd.add -text Add \
+	    -command [list tagdialog::Add $t]] \
+    	[ttk::button $cmd.remove -text Remove \
+	    -command [list tagdialog::Remove $t]] \
+    	[ttk::button $cmd.apply -text Apply \
+	    -command [list tagdialog::Apply $t]] \
+    	[ttk::button $cmd.close -text Close \
+	    -command [list destroy $t]] \
+    -padx 6 -pady 6
+    grid columnconfigure $cmd 0 -weight 1
+
+    bind $t <KeyPress-Escape> [list event generate $cmd.close <<Invoke>>]
+    bind $t <KeyPress-F5> [list tagdialog::Refresh $t]
+    if {$::havekeynav} { keynav::defaultButton $cmd.apply }
+
+    # Assemble dialog.
+    #
+    #pack $f.tag -side top -fill x
+    pack $f.cmd -side bottom -fill x
+    pack $f.options -side top -fill x
+
+    pack $f -expand true -fill both
+
+    wm deiconify $t; return $t
+}
+
+proc tagdialog::cbpost {t cb} {
+    upvar #0 $t T
+    set tv $T(treeview)
+    #@@@NYI: $cb configure -values [$tv tag names]
+    $cb configure -values [list A B C DIR FILE]
+}
+
+proc tagdialog::Refresh {t} {
+    upvar #0 $t T
+    array set T [$T(treeview) tag configure $T(tag)]
+}
+
+proc tagdialog::Apply {t} {
+    upvar #0 $t T
+    foreach option $T(options) {
+	$T(treeview) tag configure $T(tag) $option $T($option)
+    }
+}
